##################################################
#
# Estimating a base line SIENA model that later inspires
# the simulation of the data
#
# Replicating Chris Baerveldt's delinquency and friendship study
# Baerveldt, Völker, van Rossem (2008)
# "Revisiting Selection and Influence: An Inquiry into the Friendship Networks
# of High School Students and Their Association with Delinquency"
#
# ~ specified similar to results table p. 573, using school 1, 3, 4, 6
#
# Christoph Stadtfeld
# 2. September, 2013
####################################################3

library(RSiena)
library(igraph)

# Creating list variables
classIds <- c(1,3,4,6) # according to baerveldth1346.csv
classIds <- c(1, 3, 4, 6, 7, 8, 9, 10, 11, 13) # 10 classes
classIds <- c(1, 3, 4, 6, 7, 8, 9, 10, 11, 13, 14, 15, 16, 18, 19, 20, 21, 22, 23) # all 19 classes
friendship.w1 <- vector("list", 1)
friendship.w2 <- vector("list", 1)
delinquency.w1 <- vector("list", 1)
delinquency.w2 <- vector("list", 1)
coethnicity <- vector("list", 1)
gender <- vector("list", 1)
sienaData <- vector("list", 1)

# Loading data
for (i in c( 1 : length(classIds) ) ){
  friendship.w1[[i]] <- as.matrix(read.table(paste0("CB_data/N34_",classIds[i],".DAT")))
  friendship.w2[[i]] <- as.matrix(read.table(paste0("CB_data/HN34_",classIds[i],".DAT")))
  delinquency.w1[[i]] <- unlist(read.table(paste0("CB_data/cbc",classIds[i],".dat"))[,1])
  delinquency.w2[[i]] <- unlist(read.table(paste0("CB_data/cbc",classIds[i],".dat"))[,2])
  gender[[i]] <- unlist(read.table(paste0("CB_data/CBE",classIds[i],".DAT"))[,1])
  coethnicity[[i]] <- as.matrix(read.table(paste0("CB_data/cbe",classIds[i],".sim")))
}

# plot friendship networks
myplot <- function(g){
  plot(g,
       vertex.size = 3,
       vertex.label = "",
       vertex.color = "red",
       edge.arrow.size = 0.5,
       edge.color = "black"
       )
}

myplot(graph.adjacency(friendship.w1[[1]]))
myplot(graph.adjacency(friendship.w2[[1]]))
myplot(graph.adjacency(friendship.w1[[2]]))
myplot(graph.adjacency(friendship.w1[[2]]))
myplot(graph.adjacency(friendship.w1[[3]]))
myplot(graph.adjacency(friendship.w1[[3]]))
myplot(graph.adjacency(friendship.w1[[4]]))
myplot(graph.adjacency(friendship.w1[[4]]))
   
# Create SIENA data objects
for (i in c( 1 : length(classIds) ) ){
  nActors <- dim(friendship.w1[[i]])[1]
  sienaData[[i]] <- sienaDataCreate(
    friendship = sienaDependent(
      array(c(friendship.w1[[i]], friendship.w2[[i]]), dim = c(nActors, nActors, 2)),
      type="oneMode"
      ),
    delinquency = sienaDependent(
      array(c(delinquency.w1[[i]], delinquency.w2[[i]]), dim = c(nActors, 2)),
      type = "behavior"
      ),
    gender = coCovar(gender[[i]]),
    coethnicity = coDyadCovar(coethnicity[[i]])
    )
}
# Create SIENA group model
sienaDataGrouped <- sienaGroupCreate(sienaData)


# Specify SIENA model according to table 4, page 537. See reference above.
# added some parameters

# Network effects
effects <- getEffects(sienaDataGrouped)
effects <- includeEffects(effects, transTrip, cycle3, name = "friendship")
effects <- includeEffects(effects, egoX, altX, sameX, name = "friendship", interaction1 = "gender")
# ignore imortance of friends within school
effects <- includeEffects(effects, egoX, altX, sameX, name = "friendship", interaction1 = "delinquency")
effects <- includeEffects(effects, X, name = "friendship", interaction1 = "coethnicity")

# Delinquency effects
effects <- includeEffects(effects, effFrom, name = "delinquency", interaction1 = "gender")
effects <- includeEffects(effects, avAlt, name = "delinquency", interaction1 = "friendship")


# Estimate SIENA model
alg <- sienaAlgorithmCreate(projname = "friendship-delinquency-schools")

?sienaAlgorithmCreate

result <- siena07(alg, data = sienaDataGrouped, effects = effects,
                  useCluster = TRUE, nbrNodes = 4)

result <- siena07(alg, data = sienaDataGrouped, effects = effects,
                  useCluster = TRUE, nbrNodes = 4,
                  prevAns = result)
